/*
 * File:   system_init.c
 * Author: yts
 *
 * Created on 2017/10/04, 9:05
 */

// DSPIC33FJ128GP802 Configuration Bit Settings

// 'C' source line config statements


// FBS
#pragma config BWRP = WRPROTECT_OFF     // Boot Segment Write Protect (Boot Segment may be written)
#pragma config BSS = NO_FLASH           // Boot Segment Program Flash Code Protection (No Boot program Flash segment)
#pragma config RBS = NO_RAM             // Boot Segment RAM Protection (No Boot RAM)

// FSS
#pragma config SWRP = WRPROTECT_OFF     // Secure Segment Program Write Protect (Secure segment may be written)
#pragma config SSS = NO_FLASH           // Secure Segment Program Flash Code Protection (No Secure Segment)
#pragma config RSS = NO_RAM             // Secure Segment Data RAM Protection (No Secure RAM)

// FGS
#pragma config GWRP = OFF               // General Code Segment Write Protect (User program memory is not write-protected)
#pragma config GSS = OFF                // General Segment Code Protection (User program memory is not code-protected)

// FOSCSEL
#pragma config FNOSC = PRIPLL           // Oscillator Mode (Primary Oscillator (XT, HS, EC) w/ PLL)
#pragma config IESO = ON                // Internal External Switch Over Mode (Start-up device with FRC, then automatically switch to user-selected oscillator source when ready)

// FOSC
#pragma config POSCMD = XT              // Primary Oscillator Source (XT Oscillator Mode)
#pragma config OSCIOFNC = OFF           // OSC2 Pin Function (OSC2 pin has clock out function)
#pragma config IOL1WAY = ON             // Peripheral Pin Select Configuration (Allow Only One Re-configuration)
#pragma config FCKSM = CSECME           // Clock Switching and Monitor (Both Clock Switching and Fail-Safe Clock Monitor are enabled)

// FWDT
#pragma config WDTPOST = PS32768        // Watchdog Timer Postscaler (1:32,768)
#pragma config WDTPRE = PR128           // WDT Prescaler (1:128)
#pragma config WINDIS = OFF             // Watchdog Timer Window (Watchdog Timer in Non-Window mode)
#pragma config FWDTEN = OFF             // Watchdog Timer Enable (Watchdog timer enabled/disabled by user software)

// FPOR
#pragma config FPWRT = PWR128           // POR Timer Value (128ms)
#pragma config ALTI2C = OFF             // Alternate I2C  pins (I2C mapped to SDA1/SCL1 pins)

// FICD
#pragma config ICS = PGD1               // Comm Channel Select (Communicate on PGC1/EMUC1 and PGD1/EMUD1)
#pragma config JTAGEN = OFF             // JTAG Port Enable (JTAG is Disabled)

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

#include <xc.h>
#include "dsp.h"

/*=============================================================================
initDac() is used to configure D/A. 
=============================================================================*/
void initDac(void)
{
    /* Initiate DAC Clock */
    ACLKCONbits.SELACLK  = 0;  // PLL as Clock Source
    ACLKCONbits.AOSCMD   = 0;  // Auxiliary Oscillator Disabled
    ACLKCONbits.APSTSCLR = 7;  // Fvco/1 = 160 MHz/1 = 160 MHz
    ACLKCONbits.ASRCSEL  = 0;  // No effect

    // Sampling Rate Fs = DACCLK/256 = 104.1 kHz
    DAC1CONbits.DACSIDL = 0;    //Continue in Idle mode
    DAC1CONbits.AMPON   = 0;   // Analog Output Amplifier is enabled during Sleep Mode/Stop-in Idle mode
    DAC1CONbits.FORM    = 1;   // Data Format is signed integer
    DAC1CONbits.DACFDIV = 5;   // DACCLK = ACLK/(DACFDIV - 1): 160 MHz/6 = 26.7 MHz

    DAC1STATbits.ROEN = 1;	// Right Channel DAC Output Enabled
    DAC1DFLT          = 0x0000;	// DAC Default value is the midpoint
    
    DAC1CONbits.DACEN   = 1;   // DAC1 Module Enabled
}


void SYS_Initialize ( void )
{

    // Clock settings for 40MHz
    // Fosc= Fin * M / (N1 * N2), Fcy = Fosc / 2
    // Fosc= 10 MHz * 32 / (2 * 2) = 80 MHz, Fcy = Fosc/2 = 40MHz
    CLKDIVbits.PLLPRE = 0;      	// N2=2 (1,2,4,8,16,32,64,256) 0.8-8 MHz
    PLLFBD = 30;               		// M=32 (2-33) 100-200 MHz
    CLKDIVbits.PLLPOST = 0;     	// N1=2 (2,4,8) 12.5-80 MHz

    // wait PLL lock
    while(OSCCONbits.LOCK !=1 );
	
    initDac();    // Initialize the D/A converter

}


/*=============================================================================
_DAC1RInterrupt(): ISR name is chosen from the device linker script.
=============================================================================*/
void __attribute__((interrupt, no_auto_psv)) _DAC1RInterrupt(void)
{
  IFS4bits.DAC1RIF = 0; // Clear Right Channel Interrupt Flag
}
